﻿using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Text;
using Kivii.Third.Bestseller.Entities;
using Kivii.Third.Bestseller.Requests;
using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Net.Sockets;
using System.Security.Cryptography;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Kivii.Third.Bestseller.Extensions
{
    public static class OrderExtension
    {
        /// <summary>
        /// 拉取订单
        /// </summary>
        /// <param name="baseUrl"></param>
        /// <param name="apptoken"></param>
        /// <returns></returns>
        public static OrderResponse PullOrders(this JsonHttpClient _client, string apptoken, string appsecret, DateTime? beginTime = null, DateTime? endTime = null, int length = 200)
        {
            (_client == null).ThrowIfTrue("PullOrders:请传入_client");
            (apptoken.IsNullOrEmpty()).ThrowIfTrue("PullOrders:请传入apptoken");
            (appsecret.IsNullOrEmpty()).ThrowIfTrue("PullOrders:请传入appsecret");
            //var client = new JsonServiceClient(baseUrl);

            var timeStamp = GetTimeStamp();
            var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            var signature = GetSign(appSign);
            var getWtsListRequest = new BestsellerOrderGetWtsList();
            getWtsListRequest.appToken = apptoken;
            getWtsListRequest.signature = signature;
            getWtsListRequest.timestamp = timeStamp;
            getWtsListRequest.length = length;
            if (beginTime != null && beginTime.Value != DateTime.MinValue) getWtsListRequest.beginTime = beginTime.Value.ToString("yyyy-MM-dd");
            if (endTime != null && endTime.Value != DateTime.MinValue) getWtsListRequest.endTime = endTime.Value.ToString("yyyy-MM-dd");

            var response = _client.Get(getWtsListRequest);
            return response;
        }

        public static OrderResponse PullOrderData(this JsonHttpClient _client, string apptoken, string appsecret, string docNo)
        {
            (_client == null).ThrowIfTrue("PullOrderData:请传入_client");
            (apptoken.IsNullOrEmpty()).ThrowIfTrue("PullOrderData:请传入apptoken");
            (appsecret.IsNullOrEmpty()).ThrowIfTrue("PullOrderData:请传入appsecret");
            (docNo.IsNullOrEmpty()).ThrowIfTrue("PullOrderData:请传入docNo");
            //var client = new JsonServiceClient(baseUrl);

            var timeStamp = GetTimeStamp();
            var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            var signature = GetSign(appSign);
            var getWtsListRequest = new BestsellerOrderGetData();
            getWtsListRequest.appToken = apptoken;
            getWtsListRequest.signature = signature;
            getWtsListRequest.timestamp = timeStamp;
            getWtsListRequest.docNo = docNo;
            var response = _client.Get(getWtsListRequest);
            return response;
        }

        /// <summary>
        /// 确认接受三方检测单
        /// </summary>
        /// <param name="_client"></param>
        /// <param name="apptoken"></param>
        /// <param name="appsecret"></param>
        /// <param name="orderNo"></param>
        /// <returns></returns>
        public static OrderResponse AcceptOrderConfirm(this JsonHttpClient _client, string orderNo, string apptoken=null, string appsecret = null)
        {
            (_client == null).ThrowIfTrue("AcceptOrderConfirm:请传入_client");
            if (apptoken.IsNullOrEmpty()) apptoken = Configs._token;//.ThrowIfTrue("AcceptOrderConfirm:请传入apptoken");
            if (appsecret.IsNullOrEmpty()) appsecret = Configs._secret;//.ThrowIfTrue("AcceptOrderConfirm:请传入appsecret");
            (orderNo.IsNullOrEmpty()).ThrowIfTrue("AcceptOrderConfirm:请传入orderNo");
            //var client = new JsonServiceClient(baseUrl);

            var timeStamp = GetTimeStamp();
            var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            var signature = GetSign(appSign);
            var paramsString = $"?appToken={apptoken}&signature={signature}&timestamp={timeStamp}";
            var getWtsListRequest = new Dictionary<string, string>();
            getWtsListRequest.Add("ORDER_NO", orderNo);
            try
            {
                var response = _client.Post<OrderResponse>($"{Configs.RouteAcceptWts}{paramsString}", getWtsListRequest);
                return response;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public static OrderResponse RejectOrder(this JsonHttpClient _client, string orderNo, string apptoken = null, string appsecret = null)
        {
            (_client == null).ThrowIfTrue("RejectOrder:请传入_client");
            if (apptoken.IsNullOrEmpty()) apptoken = Configs._token;//.ThrowIfTrue("RejectOrder:请传入apptoken");
            if (appsecret.IsNullOrEmpty()) appsecret = Configs._secret;//.ThrowIfTrue("RejectOrder:请传入appsecret");
            (orderNo.IsNullOrEmpty()).ThrowIfTrue("RejectOrder:请传入orderNo");
            //var client = new JsonServiceClient(baseUrl);

            var timeStamp = GetTimeStamp();
            var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            var signature = GetSign(appSign);
            var paramsString = $"?appToken={apptoken}&signature={signature}&timestamp={timeStamp}";
            var getWtsListRequest = new Dictionary<string, string>();
            getWtsListRequest.Add("ORDER_NO", orderNo);
            try
            {
                var response = _client.Post<OrderResponse>($"{Configs.RouteRejectWts}{paramsString}", getWtsListRequest);
                return response;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// 创建第三方订单数据
        /// </summary>
        /// <param name="orders"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static List<ThirdReportOrder> AcceptThirdContractOrder(this List<BestsellerOrder> orders, Guid? ownerKvid = null, string ownerName = null, IDbConnection conn = null)
        {
            (orders.IsNullOrEmpty()).ThrowIfTrue("AcceptThirdContractOrder:请传入orders");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            }

            var rtns = new List<ThirdReportOrder>();

            //找出最小创建日期的订单
            var queryThirdReportOrders = conn.From<ThirdReportOrder>();
            queryThirdReportOrders.Where(o => o.ThirdType == typeof(BestsellerOrder).FullName);
            queryThirdReportOrders.Select(o => o.SerialNumber);
            var thirdReportOrders = conn.Column<string>(queryThirdReportOrders);

            try
            {
                foreach (var item in orders)
                {
                    if (thirdReportOrders.Exists(o => o == item.ORDER_NO)) continue;

                    var third = new ThirdReportOrder();
                    third.SerialNumber = item.ORDER_NO;
                    third.Name = item.MAT_NAME;
                    third.ThirdName = item.BRAND_DISPLAY;
                    third.ThirdType = typeof(BestsellerOrder).FullName;
                    third.IsAccepted = false;
                    third.IsUploaded = false;
                    third.OperateTime = DateTime.Now;

                    third.ManufacturerName = item.APPLY_ORG_NAME;
                    third.ManufacturerContactName = item.CONTACT_PERSON;
                    third.SampleName = item.MAT_NAME;
                    third.BatchNumber = item.PRD_CODE;
                    third.Brand = item.BRAND_DISPLAY;
                    //thirdReportOrder.Content = JsonSerializer.SerializeToString(item);
                    third.Metadata = new Dictionary<string, string>();
                    foreach (var propertyInfo in item.GetType().GetProperties())
                    {
                        if (propertyInfo.Name == "ITEMS")
                        {
                            var items = item.ITEMS;
                            if (items.IsNullOrEmpty()) items = new List<BestsellerOrderDetail>();
                            third.Metadata.Add(propertyInfo.Name, JsonSerializer.SerializeToString(items));
                            continue;
                        }

                        if (propertyInfo.Name == "SAMPLES")
                        {
                            var items = item.SAMPLES;
                            if (items.IsNullOrEmpty()) items = new List<BestSellerOrderSample>();
                            third.Metadata.Add(propertyInfo.Name, JsonSerializer.SerializeToString(items));
                            continue;
                        }
                        var value = propertyInfo.GetValue(item);
                        third.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                    }
                    conn.Insert(third);
                    //trans?.Commit();
                    third.RemoveAllOnlyProperties();
                    rtns.Add(third);
                }
            }
            catch (Exception ex)
            {
                //trans?.Rollback();
                throw ex;
            }

            return rtns;
        }

        /// <summary>
        /// 回传报告结果
        /// </summary>
        /// <param name="_client"></param>
        /// <param name="apptoken"></param>
        /// <param name="appsecret"></param>
        /// <param name="orderResult"></param>
        /// <returns></returns>
        public static OrderResponse UploadOrderResult(this JsonHttpClient _client, string apptoken, string appsecret, OrderResult orderResult)
        {
            (_client == null).ThrowIfTrue("UploadOrderResult:请传入_client");
            (apptoken.IsNullOrEmpty()).ThrowIfTrue("UploadOrderResult:请传入apptoken");
            (appsecret.IsNullOrEmpty()).ThrowIfTrue("UploadOrderResult:请传入appsecret");
            orderResult.ThrowIfNull("UploadOrderResult:请传入回传内容");
            var timeStamp = GetTimeStamp();
            var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            var signature = GetSign(appSign);

            var jsons = JsonSerializer.SerializeToString(orderResult);
            var body = new Dictionary<string, string>();
            body.Add("DATA", jsons);

            var paramsString = $"?appToken={apptoken}&signature={signature}&timestamp={timeStamp}";

            var response = _client.Post<OrderResponse>($"{Configs.RouteUpdateWtsResult}{paramsString}", body);
            return response;
        }

        /// <summary>
        /// 回传报告PDF
        /// </summary>
        /// <param name="_client"></param>
        /// <param name="apptoken"></param>
        /// <param name="appsecret"></param>
        /// <param name="orderNo"></param>
        /// <param name="orderPdfs"></param>
        /// <returns></returns>
        public static OrderResponse UploadOrderPdf(this JsonHttpClient _client, string apptoken, string appsecret, string orderNo, List<OrderPdf> orderPdfs)
        {
            (_client == null).ThrowIfTrue("UploadOrderPdf:请传入_client");
            (apptoken.IsNullOrEmpty()).ThrowIfTrue("UploadOrderPdf:请传入apptoken");
            (appsecret.IsNullOrEmpty()).ThrowIfTrue("UploadOrderPdf:请传入appsecret");
            orderNo.ThrowIfNullOrEmpty("请传入订单号");
            orderPdfs.ThrowIfNullOrEmpty("UploadOrderPdf:请传入回传内容");
            var timeStamp = GetTimeStamp();
            var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            var signature = GetSign(appSign);

            var jsons = JsonSerializer.SerializeToString(orderPdfs);
            var body = new Dictionary<string, string>();
            body.Add("DATA", jsons);
            body.Add("ORDER_NO", orderNo);
            var paramsString = $"?appToken={apptoken}&signature={signature}&timestamp={timeStamp}";
            Console.WriteLine($"-------PrePost:UploadOrderPdf:{JsonSerializer.SerializeToString(body)}\n");
            var response = _client.Post<OrderResponse>($"{Configs.RouteUpdateWtsReport}{paramsString}", body);
            return response;
        }

        /// <summary>
        /// 确认完成检测
        /// </summary>
        /// <param name="_client"></param>
        /// <param name="apptoken"></param>
        /// <param name="appsecret"></param>
        /// <param name="result"></param>
        /// <returns></returns>
        public static OrderResponse UploadOrderComplete(this JsonHttpClient _client, string apptoken, string appsecret, OrderResult result)
        {
            (_client == null).ThrowIfTrue("UploadOrderComplete:请传入_client");
            (apptoken.IsNullOrEmpty()).ThrowIfTrue("UploadOrderComplete:请传入apptoken");
            (appsecret.IsNullOrEmpty()).ThrowIfTrue("UploadOrderComplete:请传入appsecret");
            (result.ORDER_NO.IsNullOrEmpty()).ThrowIfTrue("UploadOrderComplete:请传入orderNo");
            var timeStamp = GetTimeStamp();
            var appSign = $"apptoken={apptoken}&appsecret={appsecret}&timestamp={timeStamp}";
            var signature = GetSign(appSign);

            var body = new Dictionary<string, string>();
            body.Add("ORDER_NO", result.ORDER_NO);
            body.Add("RPT_NO", result.RPT_ITEMS[0].RPT_NO);
            var paramsString = $"?appToken={apptoken}&signature={signature}&timestamp={timeStamp}";
            Console.WriteLine($"-------PrePost:UploadOrderComplete:{result.ORDER_NO}\n");
            var response = _client.Post<OrderResponse>($"{Configs.RouteCloseWts}{paramsString}", body);
            return response;
        }

        public static void LinkReports(List<ThirdReportOrder> thirds = null)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            if (thirds.IsNullOrEmpty())
            {
                thirds = conn.Select<ThirdReportOrder>(o => o.IsAccepted == false & o.IsOffseted == false & o.IsUploaded == false & o.ThirdType == typeof(BestsellerOrder).FullName);
            }
            if (thirds.IsNullOrEmpty()) return;
            var queryReports = conn.From<Report>();
            queryReports.Where(o => Sql.In(o.BizId, thirds.ConvertAll(p => p.SerialNumber)) & o.Status >= (int)ReportStatusType.ReportIssue).OrderBy(o => o.ReportId);
            var allReports = conn.Select(queryReports);
            if (allReports.IsNullOrEmpty()) return;

            var client = new JsonHttpClient(Configs._defaultUrl);

            foreach (var third in thirds)
            {
                var reports = allReports.Where(o => o.BizId == third.SerialNumber).ToList();
                if (reports.IsNullOrEmpty()) continue;
                //if (reports.Exists(o => o.TestJudgement == "不合格"))
                //{
                //    third.ReportIds = "存在不合格报告请手动关联!";
                //    third.AddOnlyProperties(o => o.ReportIds);
                //    conn.UpdateOnly(third);
                //    continue;
                //}
                third.ReportIds = string.Empty;
                var reportIds = new List<string>();
                foreach (var report in reports)
                {
                    report.BizId = third.SerialNumber;
                    report.AddOnlyProperties(o => o.BizId);
                    report.BizKvid = third.Kvid;
                    report.AddOnlyProperties(o => o.BizKvid);
                    report.BizType = typeof(ThirdReportOrder).FullName;
                    report.AddOnlyProperties(o => o.BizType);
                    conn.UpdateOnly(report);

                    var thirdDetail = new ThirdReportOrderDetail();
                    thirdDetail.ThirdOrderKvid = third.Kvid;
                    thirdDetail.ReportId = report.ReportId;
                    thirdDetail.ReportKvid = report.Kvid;
                    thirdDetail.ThirdType = third.ThirdType;
                    thirdDetail.SerialNumber = third.SerialNumber;
                    thirdDetail.Name = third.Name;
                    thirdDetail.ThirdName = third.ThirdName;
                    thirdDetail.IsUploaded = false;
                    thirdDetail.OperateTime = DateTime.Now;
                    thirdDetail.Metadata = new Dictionary<string, string>();
                    thirdDetail.Metadata = report.Metadata;
                    foreach (var propertyInfo in report.GetType().GetProperties())
                    {
                        if (propertyInfo.Name == "Metadata") continue;
                        try
                        {
                            var value = propertyInfo.GetValue(report);
                            thirdDetail.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                        }
                        catch { }

                    }
                    conn.Insert(thirdDetail);
                    reportIds.Add(report.ReportId);
                }

                if (third.Metadata.IsNullOrEmpty()) third.Metadata = new Dictionary<string, string>();
                third.Metadata["Amount"] = reports.Sum(o => o.Amount + o.AmountTest).ToString();
                third.AddOnlyProperties(o => o.Metadata);
                third.ReportIds = string.Join("、", reportIds);
                third.AddOnlyProperties(o => o.ReportIds);
                third.IsAccepted = true;
                third.AddOnlyProperties(o => o.IsAccepted);
                conn.UpdateOnly(third);

                try
                {
                    client.AcceptOrderConfirm(third.SerialNumber, Configs._token, Configs._secret);
                }
                catch { }
            }
        }

        public static string GetTimeStamp()
        {
            System.DateTime startTime = TimeZone.CurrentTimeZone.ToLocalTime(new System.DateTime(1970, 1, 1)); // 当地时区
            var timeStamp = ((int)(DateTime.Now - startTime).TotalSeconds).ToString(); // 相差秒数
            return timeStamp;
            //return string.Format("{0:yyyyMMddHHmmss}", DateTime.Now);//$"{DateTime.Now:yyyyMMddHHmmss}";
        }

        /// <summary>
        /// apptoken=12345&appsecret=56789&timestamp=1632127400
        /// </summary>
        /// <param name="accessKey"></param>
        /// <returns></returns>
        public static string GetSign(string accessKey)
        {
            if (accessKey.IsNullOrEmpty()) accessKey = string.Empty;
            MD5 md5 = new MD5CryptoServiceProvider();
            byte[] targetData = md5.ComputeHash(System.Text.Encoding.UTF8.GetBytes(accessKey));
            StringBuilder sign = new StringBuilder("");
            foreach (byte b in targetData)
            {
                sign.AppendFormat("{0:x2}", b);
            }
            return sign.ToString().ToUpper();
        }

        /// <summary>
        /// 字典类型转化为对象
        /// </summary>
        /// <param name="dic"></param>
        /// <returns></returns>
        public static BestsellerOrder DicToObject(this Dictionary<string, string> dic)
        {
            var md = new BestsellerOrder();
            CultureInfo cultureInfo = Thread.CurrentThread.CurrentCulture;
            TextInfo textInfo = cultureInfo.TextInfo;
            foreach (var d in dic)
            {
                var filed = textInfo.ToTitleCase(d.Key);
                try
                {
                    var value = d.Value;
                    var property = md.GetType().GetProperty(filed);
                    if (property == null) continue;
                    if (!property.PropertyType.IsGenericType)
                    {
                        switch (property.Name)
                        {
                            case "ITEMS":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<List<BestsellerOrderDetail>>(value), null);
                                break;
                            default:
                                property.SetValue(md, string.IsNullOrEmpty(value) ? null : Convert.ChangeType(value, property.PropertyType), null);
                                break;
                        }
                    }
                    else
                    {
                        Type genericTypeDefinition = property.PropertyType.GetGenericTypeDefinition();
                        switch (property.Name)
                        {
                            case "ITEMS":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<List<BestsellerOrderDetail>>(value), null);
                                break;
                            default:
                                //泛型Nullable<>
                                if (genericTypeDefinition == typeof(Nullable<>))
                                {
                                    property.SetValue(md, string.IsNullOrEmpty(value) ? null : Convert.ChangeType(value, Nullable.GetUnderlyingType(property.PropertyType)), null);
                                }
                                break;
                        }
                    }
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
            return md;
        }

    }
}
