﻿using Kivii.Lims;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Third.Bestseller.Entities;
using Kivii.Third.Bestseller.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Third.Bestseller.Jobs
{
    public  class OrderPullJob : IJob
    {
        public string Name => "拉取订单";

        public string Description => "拉取订单";

        public IJobContext TaskContext { get; set; }

        public bool Execution()
        {
            #region 设置缓存1分钟内不能连续请求
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = KiviiContext.GetUrnKey($"{typeof(BestsellerOrder).FullName}.PullOrder:LockedOneMinute");
            var cacheValue = cache.Get<string>(cacheKey);
            if (!cacheValue.IsNullOrEmpty())
            {
                throw new Exception(cacheValue);
            }
            #endregion
            var baseUrl = TaskContext.Arguments["BaseUrl"];
            baseUrl.ThrowIfNullOrEmpty("未配置参数:BaseUrl");
            var token = TaskContext.Arguments["Token"];
            token.ThrowIfNullOrEmpty("未配置参数:Token");
            var secret = TaskContext.Arguments["Secret"];
            secret.ThrowIfNullOrEmpty("未配置参数:Secret");
            var length = TaskContext.Arguments.ContainsKey("Length") ? TaskContext.Arguments["Length"].ToInt() : 200;
            var ownerKvidstr = TaskContext.Arguments["OwnerKvid"];
            var ownerKvid = Guid.Empty;
            if (!ownerKvidstr.IsNullOrEmpty()) Guid.TryParse(ownerKvidstr, out ownerKvid);
            var ownerName = TaskContext.Arguments["OwnerName"];

            try
            {
                var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
                DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime), p => p.ThirdType == typeof(BestsellerOrder).FullName);

                var client = new JsonHttpClient(baseUrl);
                var response = client.PullOrders(token, secret, maxTranslateTime);
                cache.Set(cacheKey, "一分钟内不可多次查询!", TimeSpan.FromMinutes(1));

                if (response.code != 200)
                {
                    TaskContext.Message = $"{response.message}";
                    return false;
                }
                if (response.data == null)
                {
                    TaskContext.Message = $"未获取到数据!";
                    return true;
                }
                if (response.data.results.IsNullOrEmpty())
                {
                    TaskContext.Message = $"未获取到数据!";
                    return true;
                }

                var results = response.data.results.AcceptThirdContractOrder(ownerKvid, ownerName, conn);
                TaskContext.Message = $"新增{results.Count}条!";
                //try
                //{
                //    foreach (var item in results)
                //    {
                //        client.AcceptOrderConfirm(token, secret, item.SerialNumber);
                //    }
                //}
                //catch { }
                return true;
            }
            catch (Exception ex)
            {
                TaskContext.Message = $"错误：{ex.Message}";
                return false;
            }
            finally
            {
                var thread = KiviiContext.NewThread(() =>
                {
                    //没有关联报告的 自动执行符合条件的关联
                    OrderExtension.LinkReports();
                });
                thread.Start();
            }
        }
        public void Dispose()
        {
        }
    }

    public class OrderUploadJob : IJob
    {
        public string Name => "上传订单";

        public string Description => "上传订单";

        public IJobContext TaskContext { get; set; }

        public bool Execution()
        {
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();

            try
            {
                var pbs = conn.Select<ThirdReportOrder>(o => o.ThirdType == typeof(BestsellerOrder).FullName & o.IsUploaded == false & o.IsAccepted == true);
                if (pbs.IsNullOrEmpty())
                {
                    TaskContext.Message = "暂无数据处理";
                    return true;
                }
                var pbKvids = pbs.ConvertAll(o => o.Kvid);

                var instance = Activator.CreateInstance(typeof(BestsellerOrder)) as IEntityIsThirdReportOrder;
                var successed = 0;
                var skip = 0;

                foreach (var kvid in pbKvids)
                {
                    var result = instance.OnUploadThirdReport(kvid);
                    if (result) successed++;
                    else skip++;
                }
                TaskContext.Message = $"合计待回传订单：{pbKvids.Count}条，成功回传：{successed}条，跳过{skip}条。";
                return true;
            }
            catch (Exception ex)
            {
                TaskContext.Message = $"错误：{ex.Message}";
                throw;
            }
        }
        public void Dispose()
        {
        }
    }
}
